function [xyblocks_r]=rotate(xyblocks,pin_block,n)

%load saverotate

% Landon Brockmeyer
% 6-27-14

% Take the first 60 degree rotation from ineri, rotate every point 60
% degrees and stack it on top of the first part. This takes advantage of
% the rotational symmetry of a hexagon. This routine also ensures that the
% points are stored in the appropriate location, as if the points had been
% generated by ineri.

% input:
    m=size(xyblocks,5);
    
% loaded from ess_xyblocks:
    % xyblocks   Specifies every point in the mesh for the first 30 levels
    % pin_block  For every block, specifies the pin to which it belongs
    % type_block For every block, specifies whether corner, wall or neither


nm=6*n-5; % number of distinct pins
ns=36*n-24; % total number of blocks
nsh=30*n-24; % number of blocks in hexes

% array pos gives position of each pin going around clockwise%    9  8  7
a=zeros(6*n-5,1);                                            %  10       6
newpos=a;                                                    % 11   13    5
                                                             %  12       4
a(1)=1;                                                      %    1  2  3 
for i=2:n; a(i)=a(i-1)+1; end
for i=n+1:3*n-3; a(i)=a(i-1)+2; end
for i=2*n-1:3*n-3; a(i)=a(i)+1; end
a(3*n-2)=a(3*n-3)+n;
for i=3*n-1:4*n-3; a(i)=a(i-1)-1; end
for i=4*n-2:6*n-6; a(i)=a(i-1)-2; end
for i=5*n-4:6*n-6; a(i)=a(i)-1; end
a(6*n-5)=3*n-2;
pos=a;

% Array newpos gives pin number corresponding to 60 degree turn
for i=1:nm
    p=find(pos==i,1);
    p=p+n-1;
    if p==6*n-6+n
        p=6*n-5;
    else
        if p>6*n-6
            p=p-(6*n-6);
        end
    end
    newpos(i)=pos(p);
end


% Read through pin_block, store column# in row(value).
% Now know every block's corresponding pin.
block_num=zeros(nm,7);
for i=1:ns  
    block_num(pin_block(i),find(block_num(pin_block(i),:)==0,1,'first'))=i;
end


xyblocks_r=zeros(size(xyblocks));
im=size(xyblocks,1);
jm=size(xyblocks,2);

% Moving up the height, rotate each point 60 degrees, and store it in
% the correct pin number and block number
for h=1:m 
    for block=1:ns 
        [r,c]=find(block_num==block);
        blockmax=5;
        if r==3*n-2   % Center has 6 blocks
            blockmax=6;
        end        
        newblock=c;
        % For each rotation, the first position stored rotates also, except
        % for the ones specified in this if statement
        
        if n==2 % different for 2
           if (r~=5) && (block<=nsh)
              newblock=newblock+1;
              if newblock==blockmax+1
                 newblock=1;
              end
           end
        else
           if (r<n || find(pos==r)>2*n-2) && (block<=nsh) 
               newblock=newblock+1;
               if newblock==blockmax+1
                   newblock=1;
               end
           end
        end
        % Rotate 60 degrees
             
        for i=1:im
          xold=xyblocks(i,:,block,1,h); 
          yold=xyblocks(i,:,block,2,h); 
          th=atan2(yold,xold);
          rad=(xold.^2+yold.^2).^.5;
          xynew=[rad.*cos(th+pi/3);rad.*sin(th+pi/3)];
                   
          
          xyblocks_r(i,:,block_num(newpos(r),newblock),:,h)=xynew';
        end
    end
end
 
